﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet.Expressions
{
    [System.Diagnostics.DebuggerDisplay("{ToString(),nq}")]
    public abstract class Expression<TSymbol>
    {
        public TSymbol Symbol;
        public Expression<TSymbol> Parent;
        public List<Expression<TSymbol>> Children;

        public Expression(TSymbol symbol, Expression<TSymbol> parent)
        {
            this.Parent = parent;
            this.Symbol = symbol;

            var childrenCount = GetChildrenCount(symbol);
            if (childrenCount != 0)
                this.Children = new List<Expression<TSymbol>>(childrenCount);
        }

        public Expression()
        {
            this.Parent = null;
            this.Children = new List<Expression<TSymbol>>(1);
        }

        public int MaxChildren
        {
            get { return this.Children == null ? 0 : this.Children.Capacity; }
        }

        public bool HasChildren
        {
            get { return this.Children != null; }
        }

        public bool IsComplete
        {
            get { return this.Children == null || this.Children.Count == this.Children.Capacity; }
        }

        public abstract Expression<TSymbol> GenerateChild(TSymbol symbol);
        
        public abstract int GetChildrenCount(TSymbol symbol);

        public override string ToString()
        {
            return string.Format("{0} -> {1}", this.Symbol, string.Join(", ", this.Children.Select(
                expr => expr.ToString()).ToArray()));
        }
    }
}
