﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet.Expressions
{
    [System.Diagnostics.DebuggerDisplay("Fitness = {Fitness}")]
    public sealed class ExpressionChromosome<TSymbol> : Chromosome<TSymbol>
    {
        public ExpressionChromosome(int headLength, int tailLength)
            : base()
        {
            this.HeadLength = headLength;
            this.Codons = new object[headLength + tailLength];
        }

        private ExpressionChromosome(int length)
            : base(length)
        {

        }

        private ExpressionChromosome()
            : base()
        {
        }

        public int HeadLength
        {
            get;
            private set;
        }

        public Expression<TSymbol> GenerateExpressionTree(Func<Expression<TSymbol>, TSymbol, Expression<TSymbol>> generateExpression)
        {
            Expression<TSymbol> rootExpression = null;
            var expressions = new Queue<Expression<TSymbol>>(this.HeadLength);
            Expression<TSymbol> curParentExpr = null;
            Expression<TSymbol> curExpr;
            TSymbol curCodon;

            for (int i = 0; i < this.Length; i++)
            {
                curCodon = this[i];
                //curExpr = new Expression<T>(curCodon, curParentExpr, GetChildrenCount<T>(curCodon));
                curExpr = generateExpression(curParentExpr, curCodon);
                if (curParentExpr == null)
                {
                    rootExpression = curExpr;
                    curParentExpr = rootExpression;
                    if (rootExpression.IsComplete)
                        break;
                    continue;
                }
                curParentExpr.Children.Add(curExpr);
                expressions.Enqueue(curExpr);

                while (curParentExpr.IsComplete)
                {
                    if (expressions.Count == 0)
                    {
                        i = this.Length;
                        break;
                    }
                    curParentExpr = expressions.Dequeue();
                }
            }

            return rootExpression;
        }

        public override Chromosome<TSymbol> Clone(bool copyIsOperable)
        {
            var clone = new ExpressionChromosome<TSymbol>();
            CloneBase(clone, copyIsOperable);
            clone.HeadLength = this.HeadLength;
            return clone;
        }

        public override string ToString()
        {
            return ToString(item => item.ToString());
        }

        public string ToString(Func<TSymbol, string> transform)
        {
            return string.Join(string.Empty, Array.ConvertAll(this.Codons, item => transform((TSymbol)item)));
        }
    }
}
