﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet
{
    public abstract class PairOperator : Operator
    {
        public PairOperator(Func<Population, double> probabilityFunction)
            : base(probabilityFunction)
        {
        }

        public PairOperator()
            : base()
        {
        }

        public sealed override void Perform(Population population)
        {
            var prob = this.ProbabilityFunction(population);
            var chromosomes = population.Chromosomes;
            var chosen = new bool[chromosomes.Count];
            var firstHalfCount = chromosomes.Count / 2;
            var secondHalfCount = chromosomes.Count - firstHalfCount;
            var partners = new int[secondHalfCount];
            int partnerIndex = 0;
            int chromosomeIndex;
            Chromosome chromosome;

            for (int i = 0; i < secondHalfCount; i++)
            {
                chromosomeIndex = firstHalfCount + i;
                chromosome = chromosomes[chromosomeIndex];
                if (chromosome.IsOperable)
                    partners[partnerIndex++] = chromosomeIndex;
                else
                    secondHalfCount--;
            }

            for (int i = 0; i < firstHalfCount; i++)
            {
                chromosome = chromosomes[i];
                if (chromosome.IsOperable && _rand.NextDouble() < prob)
                {
                    partnerIndex = _rand.Next(secondHalfCount--);
                    Perform(chromosome, chromosomes[partners[partnerIndex]]);
                    partners[partnerIndex] = partners[secondHalfCount];
                }
            }
        }

        public abstract void Perform(Chromosome chromosomeA, Chromosome chromosomeB);
    }
}
