﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet.Selectors
{
    public class ElitistSelector : Selector
    {
        public ElitistSelector(int elitistCount)
            : this()
        {
            this.ElitistCount = elitistCount;
        }

        public ElitistSelector()
            : base()
        {
        }

        public int ElitistCount
        {
            get;
            set;
        }

        public Func<Population, int> ElitistCountFunction
        {
            get;
            set;
        }

        public override void SelectNextGeneration(Population population, List<Chromosome> newChromosomes)
        {
            var chromosomes = population.Chromosomes;
            var count = chromosomes.Count;
            var curElitistCount = (this.ElitistCountFunction == null ? this.ElitistCount :
                this.ElitistCountFunction(population));

            var bestChromosomes = new C5.IntervalHeap<Chromosome>(curElitistCount, _fitnessComparer);
            var minBestFitness = double.MinValue;
            Chromosome chromosome;

            for (int i = 0; i < count; i++)
            {
                chromosome = chromosomes[i];
                if (bestChromosomes.Count < curElitistCount || chromosome.Fitness > minBestFitness)
                {
                    chromosome.IsOperable = false;
                    bestChromosomes.Add(chromosome);
                    if (bestChromosomes.Count > curElitistCount)
                    {
                        bestChromosomes.DeleteMin();
                        minBestFitness = bestChromosomes.FindMin().Fitness;
                    }
                    else
                    {
                        if (minBestFitness == double.MinValue || chromosome.Fitness < minBestFitness)
                            minBestFitness = chromosome.Fitness;
                    }
                }
                else
                {
                    chromosome.IsOperable = true;
                }
            }
            
            foreach (var c in bestChromosomes)
                newChromosomes.Add(c.Clone(true));
            //while (bestChromosomes.Count > 0)
            //    newChromosomes.Add(bestChromosomes.DeleteMax().Clone(true));
        }
    }
}
