﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet.Selectors
{
    public class FitnessProportionalSelector : Selector
    {
        public FitnessProportionalSelector()
            : base()
        {
        }

        public Func<double, double> FitnessTransformFunction
        {
            get;
            set;
        }

        public override void SelectNextGeneration(Population population, List<Chromosome> newChromosomes)
        {
            var chromosomes = population.Chromosomes;
            var count = chromosomes.Count;

            var cumulativeFitness = new double[count];
            var fitnessTransformFunc = this.FitnessTransformFunction;
            cumulativeFitness[0] = chromosomes[0].Fitness;
            for (int i = 1; i < count; i++)
                cumulativeFitness[i] = cumulativeFitness[i - 1] + (fitnessTransformFunc == null ? 
                    chromosomes[i].Fitness : fitnessTransformFunc(chromosomes[i].Fitness));

            var newListCapacity = newChromosomes.Capacity;
            var fitnessSum = cumulativeFitness[count - 1];
            int searchResult;
            while (newChromosomes.Count < newListCapacity)
            {
                searchResult = Array.BinarySearch(cumulativeFitness, _rand.NextDouble() * fitnessSum);
                newChromosomes.Add(chromosomes[searchResult >= 0 ? searchResult : ~searchResult].Clone(false));
            }
        }
    }
}
