﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet.Selectors
{
    public class RandomSelector : Selector
    {
        public RandomSelector()
            : this(true)
        {
        }

        public RandomSelector(bool performReplacement)
            : base()
        {
            this.PerformReplacement = performReplacement;
        }

        public bool PerformReplacement
        {
            get;
            set;
        }

        public override void SelectNextGeneration(Population population, List<Chromosome> newChromosomes)
        {
            var chromosomes = population.Chromosomes;
            var count = chromosomes.Count;

            int selectionMaxIndex = 0; // exclusive maximum
            int[] selectionIndices = null;
            if (!this.PerformReplacement)
                selectionIndices = new int[count];

            var newListCapacity = newChromosomes.Capacity;
            int curSelectionIndex;
            while (newChromosomes.Count < newListCapacity)
            {
                if (!this.PerformReplacement && selectionMaxIndex == 0)
                {
                    selectionMaxIndex = count;
                    for (int i = 0; i < selectionMaxIndex; i++)
                        selectionIndices[i] = i;
                }

                if (this.PerformReplacement)
                {
                    newChromosomes.Add(chromosomes[_rand.Next(count)].Clone(false));
                }
                else
                {
                    curSelectionIndex = _rand.Next(selectionMaxIndex);
                    newChromosomes.Add(chromosomes[selectionIndices[curSelectionIndex]].Clone(false));
                    selectionIndices[curSelectionIndex] = selectionIndices[--selectionMaxIndex];
                }
            }
        }
    }
}
