﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet.Selectors
{
    public class TournamentSelector : Selector
    {
        public TournamentSelector(int tournamentSize)
            : this(tournamentSize, 1d, true)
        {
        }

        public TournamentSelector(int tournamentSize, double chooseProbability, bool performReplacement)
            : base()
        {
            this.TournamentSize = tournamentSize;
            this.ChooseProbability = chooseProbability;
            this.PerformReplacement = performReplacement;
        }

        public int TournamentSize
        {
            get;
            set;
        }

        public double ChooseProbability
        {
            get;
            set;
        }

        public bool PerformReplacement
        {
            get;
            set;
        }

        public override void SelectNextGeneration(Population population, List<Chromosome> newChromosomes)
        {
            var chromosomes = population.Chromosomes;
            var count = chromosomes.Count;

            int selectionMaxIndex = 0; // exclusive maximum
            int[] selectionIndices = null;
            if (!this.PerformReplacement)
                selectionIndices = new int[count];

            var newListCapacity = newChromosomes.Capacity;
            var participants = new C5.IntervalHeap<Chromosome>(this.TournamentSize, _fitnessComparer);
            int curSelectionIndex;
            while (newChromosomes.Count < newListCapacity)
            {
                if (!this.PerformReplacement)
                {
                    selectionMaxIndex = count;
                    for (int i = 0; i < selectionMaxIndex; i++)
                        selectionIndices[i] = i;
                }

                for (int i = 0; i < this.TournamentSize; i++)
                {
                    if (this.PerformReplacement)
                    {
                        participants.Add(chromosomes[_rand.Next(count)]);
                    }
                    else
                    {
                        curSelectionIndex = _rand.Next(selectionMaxIndex);
                        participants.Add(chromosomes[selectionIndices[curSelectionIndex]]);
                        selectionIndices[curSelectionIndex] = selectionIndices[--selectionMaxIndex];
                    }
                }

                while (participants.Count > 0)
                {
                    if (_rand.NextDouble() < this.ChooseProbability)
                    {
                        newChromosomes.Add(participants.DeleteMax().Clone(false));
                        break;
                    }
                }
            }
        }
    }
}
