﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet
{
    public abstract class SingletonOperator : Operator
    {
        public SingletonOperator(Func<Population, double> probabilityFunction)
            : base(probabilityFunction)
        {
        }

        public SingletonOperator()
            : base()
        {
        }

        public sealed override void Perform(Population population)
        {
            var prob = this.ProbabilityFunction(population);
            var chromosomes = population.Chromosomes;
            Chromosome chromosome;

            for (int i = 0; i < chromosomes.Count; i++)
            {
                chromosome = chromosomes[i];
                if (chromosome.IsOperable && _rand.NextDouble() < prob)
                    Perform(chromosome);
            }
        }

        public abstract void Perform(Chromosome chromosome);
    }
}
