﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet
{
    public abstract class Transpositor : SingletonOperator
    {
        public Transpositor()
            : base()
        {
        }

        public sealed override void Perform(Chromosome chromosome)
        {
            int startIndex;
            int destinationIndex;
            int length;
            bool insert;

            Transpose(chromosome, out startIndex, out destinationIndex, out length, out insert);
            MoveCodons(chromosome.Codons, startIndex, destinationIndex, length, insert);
        }

        public abstract void Transpose(Chromosome chromosome, out int startIndex, out int destinationIndex,
            out int length, out bool insert);

        private void MoveCodons(object[] codons, int startIndex, int destinationIndex, int length, bool insert)
        {
            if (insert)
            {
                var tempSequence = new object[length];
                Array.Copy(codons, startIndex, tempSequence, 0, length);
                Array.Copy(codons, startIndex, codons, startIndex + length, codons.Length - startIndex - length);
                Array.Copy(tempSequence, 0, codons, destinationIndex, length);
            }
            else
            {
                Array.Copy(codons, startIndex, codons, destinationIndex, length);
            }
        }
    }
}
