﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DarwinDotNet;

namespace HelloWorld
{
    public class SimpleMutator : Mutator
    {
        public SimpleMutator()
            : base()
        {
            this.ProbabilityFunction = new Func<Population, double>(pop => 0.020d);
        }

        public override object Mutate(Chromosome chromosome, int index, object codon)
        {
            return _rand.Next(StringChromosome.Chars.Length);
        }
    }

    public class SimpleRecombinator : Recombinator
    {
        public SimpleRecombinator()
            : base()
        {
            this.ProbabilityFunction = new Func<Population, double>(pop => 1.00d);
        }

        public override void Recombine(Chromosome chromosomeA, Chromosome chromosomeB,
            out int index, out int length)
        {
            var minLength = Math.Min(chromosomeA.Length, chromosomeB.Length);

            // Double-point crossover.
            length = _rand.Next(1, minLength);
            index = _rand.Next(0, minLength - length);
        }
    }

    public class StringChromosome : Chromosome
    {
        public const string Chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ :;',.?!";

        public StringChromosome(int length)
            : base(length)
        {
        }

        protected StringChromosome()
            : base()
        {
        }

        public override Chromosome Clone(bool copyIsOperable)
        {
            var clone = new StringChromosome();
            CloneBase(clone, copyIsOperable);
            return clone;
        }

        public override string ToString()
        {
            return new string(Array.ConvertAll(this.Codons, item => (int)item < Chars.Length ?
                Chars[(int)item] : '?'));
        }
    }
}
