﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using DarwinDotNet;
using DarwinDotNet.Selectors;

namespace HelloWorld
{
    internal static class Program
    {
        internal static readonly string _target = "Hello world!";
        internal static int[] _targetCodons;
        internal static double _maxFitness = (StringChromosome.Chars.Length - 1) * _target.Length;

        internal static Random _rand = new Random();

        internal static void Main(string[] args)
        {
            var population = new Population(2048);

            _targetCodons = new int[_target.Length];
            for (int i = 0; i < _target.Length; i++)
                _targetCodons[i] = StringChromosome.Chars.IndexOf(_target[i]);

            population.GeneticOperators = new Operator[] { 
                new ElitistSelector(205),
                new FitnessProportionalSelector(),
                //new TournamentSelector(20),
                //new RankSelector(),
                new SimpleMutator(),
                new SimpleRecombinator()
            };
            population.Generate(GenerateRandomChromosome);
            population.Evolve(RunSimulation, _maxFitness);

            Console.WriteLine("Found solution in generation {0:#0}.", population.GenerationNumber);
#if DEBUG
            Console.ReadKey(true);
#endif
        }

        public static void RunSimulation(Population population)
        {
            var chromosomes = population.Chromosomes;

            //for (int i = 0; i < chromosomes.Count; i++)
            Parallel.For(0, chromosomes.Count, delegate(int i)
            {
                var chromosome = chromosomes[i];

                int sum = 0;
                for (int j = 0; j < chromosome.Length; j++)
                    sum += Math.Abs((int)chromosome[j] - _targetCodons[j]);
                chromosome.Fitness = _maxFitness - sum;
            });

            Console.WriteLine("Generation #{0:#00000} | Best Fitness: {1:#00000} | Average Fitness: {2:#000000}",
                population.GenerationNumber, population.Chromosomes.Max(c => c.Fitness),
                population.Chromosomes.Average(c => c.Fitness));
        }

        public static Chromosome GenerateRandomChromosome(Population population)
        {
            var chromosome = new StringChromosome(_target.Length);
            for (int i = 0; i < chromosome.Length; i++)
                chromosome[i] = _rand.Next(StringChromosome.Chars.Length);

            return chromosome;
        }
    }
}
