﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DarwinDotNet;
using DarwinDotNet.Expressions;

namespace SymbolicRegression
{
    public class SimpleMutator : Mutator
    {
        public SimpleMutator()
            : base()
        {
        }

        public override object Mutate(Chromosome chromosome, int index, object codon)
        {
            var exprChromsome = (ExpressionChromosome<MathSymbol>)chromosome;
            if (index < exprChromsome.HeadLength)
                return chromosome[index] = _rand.Next(0, (int)MathSymbol.Max);
            else
                return _rand.Next((int)MathSymbol.FunctionMax, (int)MathSymbol.Max);
        }
    }

    public class SimpleTranspositor : Transpositor
    {
        public SimpleTranspositor()
            : base()
        {
        }

        public override void Transpose(Chromosome chromosome, out int startIndex, out int destinationIndex,
            out int length, out bool insert)
        {
            length = _rand.Next(1, 4);
            startIndex = _rand.Next(0, chromosome.Length - length);
            destinationIndex = _rand.Next(1, chromosome.Length - length);
            insert = false;
        }
    }

    public class SimpleRecombinator : Recombinator
    {
        public SimpleRecombinator()
            : base()
        {
        }

        public override void Recombine(Chromosome chromosomeA, Chromosome chromosomeB,
            out int index, out int length)
        {
            var minLength = Math.Min(chromosomeA.Length, chromosomeB.Length);

            // Double-point crossover
            length = _rand.Next(1, minLength);
            index = _rand.Next(0, minLength - length);
        }
    }

    public class MathExpression : Expression<MathSymbol>
    {
        public MathExpression(MathSymbol symbol, Expression<MathSymbol> parent)
            : base(symbol, parent)
        {
        }

        public MathExpression()
            : base()
        {
        }

        public double Evaluate(double x, double y, double z)
        {
            if (!this.IsComplete)
                return double.NaN;

            switch (this.Symbol)
            {
                case MathSymbol.Add:
                    return ((MathExpression)this.Children[0]).Evaluate(x, y, z) +
                        ((MathExpression)this.Children[1]).Evaluate(x, y, z);
                case MathSymbol.Subtract:
                    return ((MathExpression)this.Children[0]).Evaluate(x, y, z) -
                        ((MathExpression)this.Children[1]).Evaluate(x, y, z);
                case MathSymbol.Multiply:
                    return ((MathExpression)this.Children[0]).Evaluate(x, y, z) *
                        ((MathExpression)this.Children[1]).Evaluate(x, y, z);
                case MathSymbol.Divide:
                    return ((MathExpression)this.Children[0]).Evaluate(x, y, z) /
                        ((MathExpression)this.Children[1]).Evaluate(x, y, z);
                case MathSymbol.X:
                    return x;
                case MathSymbol.Y:
                    return y;
                case MathSymbol.Z:
                    return z;
            }

            return double.NaN;
        }


        public override Expression<MathSymbol> GenerateChild(MathSymbol symbol)
        {
            return new MathExpression(symbol, this);
        }

        public override int GetChildrenCount(MathSymbol symbol)
        {
            switch (symbol)
            {
                case MathSymbol.Add:
                    return 2;
                case MathSymbol.Subtract:
                    return 2;
                case MathSymbol.Multiply:
                    return 2;
                case MathSymbol.Divide:
                    return 2;
            }
            return 0;
        }
    }

    public enum MathSymbol
    {
        Max = 7,
        FunctionMax = 4,
        Add = 0,
        Subtract = 1,
        Multiply = 2,
        Divide = 3,
        X = 4,
        Y = 5,
        Z = 6
    }
}
